@@ -46,6 +46,7 @@ from devloop.core.transactional_io import initialize_transaction_system
 from .commands import audit as audit_cmd
 from .commands import custom_agents as custom_agents_cmd
 from .commands import feedback as feedback_cmd
+from .commands import insights as insights_cmd
 from .commands import marketplace as marketplace_cmd
 from .commands import metrics as metrics_cmd
 from .commands import release as release_cmd
@@ -53,23 +54,24 @@ from .commands import summary as summary_cmd
 from .commands import telemetry as telemetry_cmd
 from .commands import tools as tools_cmd
 
-app = typer.Typer(
+_typer_app = typer.Typer(
     help="DevLoop - Development workflow automation", add_completion=False
 )
 console = Console()
 
-app.add_typer(summary_cmd.app, name="summary")
-app.add_typer(custom_agents_cmd.app, name="custom")
-app.add_typer(feedback_cmd.app, name="feedback")
-app.add_typer(marketplace_cmd.app, name="agent")
-app.add_typer(metrics_cmd.app, name="metrics")
-app.add_typer(release_cmd.app, name="release")
-app.add_typer(telemetry_cmd.app, name="telemetry")
-app.add_typer(tools_cmd.app, name="tools")
+_typer_app.add_typer(summary_cmd.app, name="summary")
+_typer_app.add_typer(custom_agents_cmd.app, name="custom")
+_typer_app.add_typer(feedback_cmd.app, name="feedback")
+_typer_app.add_typer(insights_cmd.app, name="insights")
+_typer_app.add_typer(marketplace_cmd.app, name="agent")
+_typer_app.add_typer(metrics_cmd.app, name="metrics")
+_typer_app.add_typer(release_cmd.app, name="release")
+_typer_app.add_typer(telemetry_cmd.app, name="telemetry")
+_typer_app.add_typer(tools_cmd.app, name="tools")
 
 # Wrap Typer app to handle Click-based audit command
 # Note: We can't use add_typer with Click groups due to Typer version compatibility
-_original_app = app
+_original_app = _typer_app
 
 
 class _WrappedApp:
@@ -392,18 +394,25 @@ async def _cleanup_old_data(context_store, event_store, interval_minutes: int =
             console.print(f"[yellow]Cleanup error: {e}[/yellow]")
 
 
-async def watch_async(path: Path, config_path: Path | None):
-    """Async watch implementation."""
-    # Initialize error handler
+def _load_watch_config(path: Path, config_path: Path | None) -> ConfigWrapper:
+    """Load and validate watch configuration.
+
+    Args:
+        path: Project directory path
+        config_path: Optional explicit config file path
+
+    Returns:
+        ConfigWrapper instance with loaded configuration
+
+    Raises:
+        SystemExit: On configuration errors (handled by error_handler)
+    """
     error_handler = get_error_handler()
 
-    # Load configuration with fail-fast on errors
     try:
         if config_path:
-            # Ensure it's a Path object and convert to string
             config_manager = Config(str(Path(config_path).resolve()))
         else:
-            # Default to project .devloop/agents.json
             config_path_str = str((path / ".devloop" / "agents.json").resolve())
             if not Path(config_path_str).exists():
                 error_handler.handle_startup_error(
@@ -411,12 +420,11 @@ async def watch_async(path: Path, config_path: Path | None):
                     f"Configuration file not found: {config_path_str}",
                     severity=ErrorSeverity.CRITICAL,
                     details="Run 'devloop init' to create a default configuration.",
-                    suggested_action="devloop init",
                 )
             config_manager = Config(config_path_str)
 
         config_dict = config_manager.load()
-        config = ConfigWrapper(config_dict)
+        return ConfigWrapper(config_dict)
     except ValueError as e:
         error_handler.handle_startup_error(
             ErrorCode.CONFIG_INVALID,
@@ -424,6 +432,7 @@ async def watch_async(path: Path, config_path: Path | None):
             exception=e,
             severity=ErrorSeverity.CRITICAL,
         )
+        raise  # Unreachable but satisfies type checker
     except Exception as e:
         error_handler.handle_startup_error(
             ErrorCode.CONFIG_INVALID,
@@ -431,6 +440,126 @@ async def watch_async(path: Path, config_path: Path | None):
             exception=e,
             severity=ErrorSeverity.CRITICAL,
         )
+        raise  # Unreachable but satisfies type checker
+
+
+# Agent registry: maps agent names to (class, default_triggers, uses_name_param)
+_AGENT_REGISTRY: dict[str, tuple[type, list[str], bool]] = {
+    "linter": (LinterAgent, ["file:modified"], True),
+    "formatter": (FormatterAgent, ["file:modified"], True),
+    "test-runner": (TestRunnerAgent, ["file:modified"], True),
+    "agent-health-monitor": (AgentHealthMonitorAgent, ["agent:*:completed"], True),
+    "type-checker": (TypeCheckerAgent, [], False),
+    "security-scanner": (SecurityScannerAgent, [], False),
+    "git-commit-assistant": (GitCommitAssistantAgent, [], False),
+    "performance-profiler": (PerformanceProfilerAgent, [], False),
+    "snyk": (SnykAgent, ["file:modified", "file:created"], True),
+    "code-rabbit": (CodeRabbitAgent, ["file:modified", "file:created"], True),
+}
+
+
+def _register_agents(
+    config: ConfigWrapper, event_bus: EventBus, agent_manager: AgentManager
+) -> None:
+    """Register all enabled agents based on configuration.
+
+    Uses a data-driven approach to reduce code duplication and complexity.
+
+    Args:
+        config: Configuration wrapper with agent settings
+        event_bus: Event bus for agent communication
+        agent_manager: Agent manager to register agents with
+    """
+    for agent_name, (
+        agent_class,
+        default_triggers,
+        uses_name_param,
+    ) in _AGENT_REGISTRY.items():
+        if not config.is_agent_enabled(agent_name):
+            continue
+
+        agent_config = config.get_agent_config(agent_name) or {}
+        triggers = agent_config.get("triggers", default_triggers)
+        inner_config = agent_config.get("config", {})
+
+        if uses_name_param:
+            agent = agent_class(
+                name=agent_name,
+                triggers=triggers,
+                event_bus=event_bus,
+                config=inner_config,
+            )
+        else:
+            agent = agent_class(config=inner_config, event_bus=event_bus)
+
+        agent_manager.register(agent)
+
+
+async def _initialize_stores(path: Path) -> None:
+    """Initialize context and event stores.
+
+    Args:
+        path: Project directory path
+    """
+    context_store.context_dir = path / ".devloop" / "context"
+    await context_store.initialize()
+
+    event_store.db_path = path / ".devloop" / "events.db"
+    await event_store.initialize()
+
+    console.print(f"[dim]Context store: {context_store.context_dir}[/dim]")
+    console.print(f"[dim]Event store: {event_store.db_path}[/dim]")
+
+
+async def _replay_events(event_bus: EventBus, agent_manager: AgentManager) -> None:
+    """Replay missed events and report gaps.
+
+    Args:
+        event_bus: Event bus for event replay
+        agent_manager: Agent manager with registered agents
+    """
+    replayer = EventReplayer(event_bus, agent_manager)
+    replay_stats = await replayer.replay_all_agents()
+
+    if replay_stats["total_replayed"] > 0:
+        console.print(
+            f"\n[cyan]Event Replay[/cyan]: Replayed {replay_stats['total_replayed']} missed events"
+        )
+        for agent_name, count in replay_stats["agents"].items():
+            if count > 0:
+                console.print(f"  • {agent_name}: {count} events")
+
+    if replay_stats["gaps"]:
+        console.print(
+            f"\n[yellow]⚠ Event Gaps Detected[/yellow]: {len(replay_stats['gaps'])} gaps in sequence"
+        )
+        console.print("[dim]Run 'devloop debug' for details[/dim]")
+
+
+async def _wait_for_shutdown() -> None:
+    """Wait for shutdown signal (SIGINT or SIGTERM)."""
+    shutdown_event = asyncio.Event()
+
+    def signal_handler(sig, frame):
+        shutdown_event.set()
+
+    signal.signal(signal.SIGINT, signal_handler)
+    signal.signal(signal.SIGTERM, signal_handler)
+
+    await shutdown_event.wait()
+
+
+async def watch_async(path: Path, config_path: Path | None) -> None:
+    """Async watch implementation.
+
+    Watches the project directory for file changes and triggers agents accordingly.
+
+    Args:
+        path: Project directory to watch
+        config_path: Optional path to configuration file
+    """
+    # Load configuration
+    config = _load_watch_config(path, config_path)
 
     # Create event bus
     event_bus = EventBus()
@@ -438,15 +567,8 @@ async def watch_async(path: Path, config_path: Path | None):
     # Initialize transaction system (recovery and self-healing)
     initialize_transaction_system(path / ".devloop")
 
-    # Initialize context store
-    context_store.context_dir = path / ".devloop" / "context"
-    await context_store.initialize()
-
-    # Initialize event store
-    event_store.db_path = path / ".devloop" / "events.db"
-    await event_store.initialize()
-    console.print(f"[dim]Context store: {context_store.context_dir}[/dim]")
-    console.print(f"[dim]Event store: {event_store.db_path}[/dim]")
+    # Initialize stores
+    await _initialize_stores(path)
 
     # Get global config for resource limits
     global_config = config.get_global_config()
@@ -467,96 +589,8 @@ async def watch_async(path: Path, config_path: Path | None):
     health_check = DaemonHealthCheck(path, heartbeat_interval=30)
     await health_check.start()
 
-    # Create and register agents based on configuration
-    if config.is_agent_enabled("linter"):
-        linter_config = config.get_agent_config("linter") or {}
-        linter = LinterAgent(
-            name="linter",
-            triggers=linter_config.get("triggers", ["file:modified"]),
-            event_bus=event_bus,
-            config=linter_config.get("config", {}),
-        )
-        agent_manager.register(linter)
-
-    if config.is_agent_enabled("formatter"):
-        formatter_config = config.get_agent_config("formatter") or {}
-        formatter = FormatterAgent(
-            name="formatter",
-            triggers=formatter_config.get("triggers", ["file:modified"]),
-            event_bus=event_bus,
-            config=formatter_config.get("config", {}),
-        )
-        agent_manager.register(formatter)
-
-    if config.is_agent_enabled("test-runner"):
-        test_config = config.get_agent_config("test-runner") or {}
-        test_runner = TestRunnerAgent(
-            name="test-runner",
-            triggers=test_config.get("triggers", ["file:modified"]),
-            event_bus=event_bus,
-            config=test_config.get("config", {}),
-        )
-        agent_manager.register(test_runner)
-
-    if config.is_agent_enabled("agent-health-monitor"):
-        monitor_config = config.get_agent_config("agent-health-monitor") or {}
-        health_monitor = AgentHealthMonitorAgent(
-            name="agent-health-monitor",
-            triggers=monitor_config.get("triggers", ["agent:*:completed"]),
-            event_bus=event_bus,
-            config=monitor_config.get("config", {}),
-        )
-        agent_manager.register(health_monitor)
-
-    if config.is_agent_enabled("type-checker"):
-        type_config = config.get_agent_config("type-checker") or {}
-        type_checker = TypeCheckerAgent(
-            config=type_config.get("config", {}), event_bus=event_bus
-        )
-        agent_manager.register(type_checker)
-
-    if config.is_agent_enabled("security-scanner"):
-        security_config = config.get_agent_config("security-scanner") or {}
-        security_scanner = SecurityScannerAgent(
-            config=security_config.get("config", {}), event_bus=event_bus
-        )
-        agent_manager.register(security_scanner)
-
-    if config.is_agent_enabled("git-commit-assistant"):
-        commit_config = config.get_agent_config("git-commit-assistant") or {}
-        commit_assistant = GitCommitAssistantAgent(
-            config=commit_config.get("config", {}), event_bus=event_bus
-        )
-        agent_manager.register(commit_assistant)
-
-    if config.is_agent_enabled("performance-profiler"):
-        perf_config = config.get_agent_config("performance-profiler") or {}
-        performance_profiler = PerformanceProfilerAgent(
-            config=perf_config.get("config", {}), event_bus=event_bus
-        )
-        agent_manager.register(performance_profiler)
-
-    if config.is_agent_enabled("snyk"):
-        snyk_config = config.get_agent_config("snyk") or {}
-        snyk_agent = SnykAgent(
-            name="snyk",
-            triggers=snyk_config.get("triggers", ["file:modified", "file:created"]),
-            event_bus=event_bus,
-            config=snyk_config.get("config", {}),
-        )
-        agent_manager.register(snyk_agent)
-
-    if config.is_agent_enabled("code-rabbit"):
-        code_rabbit_config = config.get_agent_config("code-rabbit") or {}
-        code_rabbit_agent = CodeRabbitAgent(
-            name="code-rabbit",
-            triggers=code_rabbit_config.get(
-                "triggers", ["file:modified", "file:created"]
-            ),
-            event_bus=event_bus,
-            config=code_rabbit_config.get("config", {}),
-        )
-        agent_manager.register(code_rabbit_agent)
+    # Register all enabled agents
+    _register_agents(config, event_bus, agent_manager)
 
     # Start everything
     await fs_collector.start()
@@ -566,37 +600,13 @@ async def watch_async(path: Path, config_path: Path | None):
     for agent_name in agent_manager.list_agents():
         console.print(f"  • [cyan]{agent_name}[/cyan]")
 
-    # Replay missed events (event persistence and recovery)
-    replayer = EventReplayer(event_bus, agent_manager)
-    replay_stats = await replayer.replay_all_agents()
-
-    if replay_stats["total_replayed"] > 0:
-        console.print(
-            f"\n[cyan]Event Replay[/cyan]: Replayed {replay_stats['total_replayed']} missed events"
-        )
-        for agent_name, count in replay_stats["agents"].items():
-            if count > 0:
-                console.print(f"  • {agent_name}: {count} events")
-
-    if replay_stats["gaps"]:
-        console.print(
-            f"\n[yellow]⚠ Event Gaps Detected[/yellow]: {len(replay_stats['gaps'])} gaps in sequence"
-        )
-        console.print("[dim]Run 'devloop debug' for details[/dim]")
+    # Replay missed events
+    await _replay_events(event_bus, agent_manager)
 
     console.print("\n[dim]Waiting for file changes... (Ctrl+C to stop)[/dim]\n")
 
     # Wait for shutdown signal
-    shutdown_event = asyncio.Event()
-
-    def signal_handler(sig, frame):
-        shutdown_event.set()
-
-    signal.signal(signal.SIGINT, signal_handler)
-    signal.signal(signal.SIGTERM, signal_handler)
-
-    # Keep running until shutdown
-    await shutdown_event.wait()
+    await _wait_for_shutdown()
 
     # Stop everything
     cleanup_task.cancel()
@@ -1011,6 +1021,71 @@ If you need to modify this file, use manual editing or ask the user.
 EOF
     exit 2
 fi
+exit 0
+""",
+        "check-devloop-context": """#!/bin/bash
+#
+# PreToolUse hook: Check for unreviewed DevLoop findings before file operations
+#
+# Fast (<100ms) hook that warns about agent findings before file ops.
+# Includes debouncing (warns at most once per 30 seconds) to avoid spam.
+#
+PROJECT_DIR="${CLAUDE_PROJECT_DIR:-.}"
+cd "$PROJECT_DIR" || exit 0
+
+# Debounce settings
+DEBOUNCE_SECONDS=30
+DEBOUNCE_FILE="/tmp/devloop-context-warned-${PPID:-$$}"
+
+# Check if we've warned recently
+if [[ -f "$DEBOUNCE_FILE" ]]; then
+    LAST_WARN=$(cat "$DEBOUNCE_FILE" 2>/dev/null || echo "0")
+    NOW=$(date +%s)
+    ELAPSED=$((NOW - LAST_WARN))
+    if [[ $ELAPSED -lt $DEBOUNCE_SECONDS ]]; then
+        exit 0
+    fi
+fi
+
+# Consume stdin
+cat > /dev/null
+
+# Fast check: Does context exist? Prefer .last_update marker (fastest)
+CONTEXT_DIR="$PROJECT_DIR/.devloop/context"
+MARKER_FILE="$CONTEXT_DIR/.last_update"
+CONTEXT_INDEX="$CONTEXT_DIR/index.json"
+
+if [[ -f "$MARKER_FILE" ]]; then
+    CHECK_FILE="$MARKER_FILE"
+elif [[ -f "$CONTEXT_INDEX" ]]; then
+    CHECK_FILE="$CONTEXT_INDEX"
+else
+    exit 0
+fi
+
+# Fast check: Is context stale (>30 min)?
+if [[ "$OSTYPE" == "darwin"* ]]; then
+    LAST_UPDATED=$(stat -f %m "$CHECK_FILE" 2>/dev/null || echo "0")
+else
+    LAST_UPDATED=$(stat -c %Y "$CHECK_FILE" 2>/dev/null || echo "0")
+fi
+NOW=$(date +%s)
+AGE_SECONDS=$((NOW - LAST_UPDATED))
+if [[ $AGE_SECONDS -gt 1800 ]]; then
+    exit 0
+fi
+
+# Quick grep for check_now count
+CHECK_NOW_COUNT=$(grep -o '"check_now"[^}]*"count": [0-9]*' "$CONTEXT_INDEX" 2>/dev/null | grep -o '[0-9]*$' || echo "0")
+
+if [[ "$CHECK_NOW_COUNT" -gt 0 ]]; then
+    date +%s > "$DEBOUNCE_FILE"
+    cat >&2 <<EOF
+DevLoop: $CHECK_NOW_COUNT agent finding(s) need attention.
+Run /agent-summary for details before continuing.
+EOF
+fi
+
 exit 0
 """,
     }
@@ -1026,6 +1101,90 @@ exit 0
     return hooks_created
 
 
+def _create_claude_settings_json(path: Path) -> bool:
+    """Create project-level .claude/settings.json with hook registrations.
+
+    This registers hooks at the project level so they work automatically
+    when Claude Code is used in this project.
+
+    Args:
+        path: Project root directory
+
+    Returns:
+        True if settings.json was created or updated, False if unchanged
+    """
+    import json
+
+    claude_dir = path / ".claude"
+    claude_dir.mkdir(parents=True, exist_ok=True)
+    settings_file = claude_dir / "settings.json"
+
+    # Hook configuration using relative paths from project root
+    hooks_config = {
+        "hooks": {
+            "SessionStart": {
+                "hooks": [
+                    {
+                        "type": "command",
+                        "matcher": "startup",
+                        "command": ".agents/hooks/claude-session-start",
+                    }
+                ]
+            },
+            "Stop": {
+                "hooks": [
+                    {
+                        "type": "command",
+                        "command": ".agents/hooks/claude-stop",
+                    }
+                ]
+            },
+            "PreToolUse": {
+                "hooks": [
+                    {
+                        "type": "command",
+                        "matcher": "Write|Edit",
+                        "command": ".agents/hooks/claude-file-protection",
+                    },
+                    {
+                        "type": "command",
+                        "matcher": "Write|Edit",
+                        "command": ".agents/hooks/check-devloop-context",
+                    },
+                ]
+            },
+        }
+    }
+
+    # Load existing settings if present
+    existing_settings = {}
+    if settings_file.exists():
+        try:
+            with open(settings_file) as f:
+                existing_settings = json.load(f)
+        except (json.JSONDecodeError, OSError):
+            pass
+
+    # Merge hooks into existing settings (don't overwrite other settings)
+    if "hooks" not in existing_settings:
+        existing_settings["hooks"] = {}
+
+    # Only add hooks that don't already exist
+    changed = False
+    for hook_name, hook_config in hooks_config["hooks"].items():
+        if hook_name not in existing_settings["hooks"]:
+            existing_settings["hooks"][hook_name] = hook_config
+            changed = True
+
+    if changed:
+        with open(settings_file, "w") as f:
+            json.dump(existing_settings, f, indent=2)
+            f.write("\n")
+        return True
+
+    return False
+
+
 def _setup_claude_hooks(
     path: Path, agents_hooks_dir: Path, non_interactive: bool
 ) -> None:
@@ -1066,6 +1225,13 @@ def _setup_claude_hooks(
     else:
         console.print("\n[green]✓[/green] Claude Code hooks already exist")
 
+    # Create project-level .claude/settings.json with hook registrations
+    settings_created = _create_claude_settings_json(path)
+    if settings_created:
+        console.print(
+            "[green]✓[/green] Created .claude/settings.json with hook registrations"
+        )
+
 
 @app.command()
 def init(
